@@ -9,7 +9,7 @@ use POSIX qw(locale_h);
 setlocale(LC_NUMERIC,"C");
 
 
-$VERSION = '22.1';
+$VERSION = '23.0';
 
 # Some function might be excluded from export and assessment.
 our @EXCLUDED_FUNCTION = ('SQUIRREL_GET_ERROR_OFFSET');
@@ -165,6 +165,7 @@ Returns a handle to a DB query statement.
 sub _table_info
 {
 	my $self = shift;
+	my $do_real_row_count = shift;
 
 	# First register all tablespace/table in memory from this database
 	my %tbspname = ();
@@ -204,7 +205,8 @@ sub _table_info
 	$sql .= $self->limit_to_objects('TABLE', 'TABLE_NAME');
 	$sth = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-	while (my $row = $sth->fetch) {
+	while (my $row = $sth->fetch)
+	{
 		$row->[2] =~ s/^BASE //;
 		$comments{$row->[0]}{comment} = $row->[1];
 		$comments{$row->[0]}{table_type} = $row->[2];
@@ -219,10 +221,12 @@ sub _table_info
 		$tables_infos{$row->[0]}{tablespace} = $tbspname{$row->[0]} || '';
 
 		# Get creation option unavailable in information_schema
-		if ($row->[6] eq 'FEDERATED') {
-			my $sth2 = $self->{dbh}->prepare("SHOW CREATE TABLE $row->[0]") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0);
+		if ($row->[6] eq 'FEDERATED')
+		{
+			my $sth2 = $self->{dbh}->prepare("SHOW CREATE TABLE `$row->[0]`") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0);
 			$sth2->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-			while (my $r = $sth2->fetch) {
+			while (my $r = $sth2->fetch)
+			{
 				if ($r->[1] =~ /CONNECTION='([^']+)'/) {
 					$tables_infos{$row->[0]}{connection} = $1;
 				}
@@ -230,6 +234,16 @@ sub _table_info
 			}
 			$sth2->finish();
 		}
+		if ($do_real_row_count)
+		{
+			$self->logit("DEBUG: looking for real row count for table $row->[0] (aka using count(*))...\n", 1);
+			$sql = "SELECT COUNT(*) FROM `$row->[0]`";
+			my $sth2 = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+			$sth2->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+			my $size = $sth2->fetch();
+			$sth2->finish();
+			$tables_infos{$row->[0]}{num_rows} = $size->[0];
+		}
 	}
 	$sth->finish();
 
@@ -358,13 +372,15 @@ sub _get_indexes
 	my %index_tablespace = ();
 
 	# Retrieve all indexes for the given table
-	foreach my $t (keys %tables_infos) {
-		my $sth = $self->{dbh}->prepare("SHOW INDEX FROM $t $condition;") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	foreach my $t (keys %tables_infos)
+	{
+		my $sql = "SHOW INDEX FROM `$t` $condition";
+		my $sth = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 		my $i = 1;
-		while (my $row = $sth->fetch) {
-
+		while (my $row = $sth->fetch)
+		{
 			next if ($row->[2] eq 'PRIMARY');
 		#Table : The name of the table.
 		#Non_unique : 0 if the index cannot contain duplicates, 1 if it can.
@@ -428,13 +444,15 @@ sub _count_indexes
 	my %data = ();
 
 	# Retrieve all indexes for the given table
-	foreach my $t (keys %tables_infos) {
-		my $sth = $self->{dbh}->prepare("SHOW INDEX FROM $t $condition;") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	foreach my $t (keys %tables_infos)
+	{
+		my $sql = "SHOW INDEX FROM `$t` $condition";
+		my $sth = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 		my $i = 1;
-		while (my $row = $sth->fetch) {
-
+		while (my $row = $sth->fetch)
+		{
 		#Table : The name of the table.
 		#Non_unique : 0 if the index cannot contain duplicates, 1 if it can.
 		#Key_name : The name of the index. If the index is the primary key, the name is always PRIMARY.
@@ -448,7 +466,6 @@ sub _count_indexes
 		#Index_type : The index method used (BTREE, FULLTEXT, HASH, RTREE).
 		#Comment : Information about the index not described in its own column, such as disabled if the index is disabled. 
 			push(@{$data{$row->[0]}{$row->[2]}}, $row->[4]);
-
 		}
 	}
 
@@ -628,12 +645,15 @@ sub _unique_key
 		%tables_infos = Ora2Pg::MySQL::_table_info($self);
 	}
 	# Retrieve all indexes for the given table
-	foreach my $t (keys %tables_infos) {
-		my $sth = $self->{dbh}->prepare("SHOW INDEX FROM $t $condition;") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	foreach my $t (keys %tables_infos)
+	{
+		my $sql = "SHOW INDEX FROM `$t` $condition";
+		my $sth = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 		my $i = 1;
-		while (my $row = $sth->fetch) {
+		while (my $row = $sth->fetch)
+		{
 			# Exclude non unique constraints
 			next if ($row->[1]);
 		#Table : The name of the table.
@@ -658,7 +678,8 @@ sub _unique_key
 			next if (!grep(/^'$type'$/, @accepted_constraint_types));
 			my $generated = 0;
 			$generated = 'GENERATED NAME' if ($row->[2] ne 'PRIMARY');
-			if (!exists $result{$row->[0]}{$idxname}) {
+			if (!exists $result{$row->[0]}{$idxname})
+			{
 				my %constraint = (type => $type, 'generated' => $generated, 'index_name' => $idxname, columns => [ ($row->[4]) ] );
 				$result{$row->[0]}{$idxname} = \%constraint if ($row->[4]);
 				$i++ if ($row->[2] ne 'PRIMARY');
@@ -1403,8 +1424,10 @@ sub _get_objects
 	}
 	$sth->finish();
 	# INDEX
-	foreach my $t (@{$infos{TABLE}}) {
-		$sth = $self->{dbh}->prepare("SHOW INDEX FROM $t->{name} FROM $self->{schema}") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	foreach my $t (@{$infos{TABLE}})
+	{
+		my $sql = "SHOW INDEX FROM `$t->{name}` FROM $self->{schema}";
+		$sth = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		while (my @row = $sth->fetchrow()) {
 			next if ($row[2] eq 'PRIMARY');
@@ -1681,17 +1704,17 @@ sub _count_sequences
 	# CREATE_OPTIONS  | varchar(255)        | YES  |     | NULL    |       |
 	# TABLE_COMMENT   | varchar(2048)       | NO   |     |         |       |
 
-	my @seqs = ();
-	my $sql = "SELECT TABLE_NAME, AUTO_INCREMENT FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='BASE TABLE' AND TABLE_SCHEMA = '$self->{schema}'";
+	my %seqs = ();
+	my $sql = "SELECT TABLE_NAME, AUTO_INCREMENT FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='BASE TABLE' AND TABLE_SCHEMA = '$self->{schema}' AND AUTO_INCREMENT IS NOT NULL";
 	$sql .= $self->limit_to_objects('TABLE', 'TABLE_NAME');
 	my $sth = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while (my $row = $sth->fetch) {
-		push(@seqs, $row->[0]) if ($row->[1]);
+		push(@{$seqs{$row->[0]}}, @$row);
 	}
 	$sth->finish();
 
-	return \@seqs;
+	return \%seqs;
 }
 
 sub _column_attributes
@@ -1732,25 +1755,33 @@ sub _column_attributes
 	# PRIVILEGES               | varchar(80)         | NO   |     |         |       |
 	# COLUMN_COMMENT           | varchar(1024)       | NO   |     |         |       |
 
-	my $sth = $self->{dbh}->prepare(<<END);
-SELECT COLUMN_NAME, IS_NULLABLE, COLUMN_DEFAULT, TABLE_NAME
+	my $sql = qq{SELECT COLUMN_NAME, IS_NULLABLE,
+	(CASE WHEN COLUMN_DEFAULT IS NOT NULL THEN COLUMN_DEFAULT ELSE EXTRA END) AS COLUMN_DEFAULT,
+	TABLE_NAME, DATA_TYPE, ORDINAL_POSITION
 FROM INFORMATION_SCHEMA.COLUMNS
 $condition
 ORDER BY ORDINAL_POSITION
-END
+};
+
+	if ($self->{db_version} < '5.5.0') {
+		$sql =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
+	}
+	my $sth = $self->{dbh}->prepare($sql);
 	if (!$sth) {
 		$self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	}
 	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %data = ();
-	while (my $row = $sth->fetch) {
+	while (my $row = $sth->fetch)
+	{
 		$data{$row->[3]}{"$row->[0]"}{nullable} = $row->[1];
 		$data{$row->[3]}{"$row->[0]"}{default} = $row->[2];
+		# Store the data type of the column following its position
+		$data{$row->[3]}{data_type}{$row->[5]} = $row->[4];
 	}
 
 	return %data;
-
 }
 
 sub _list_triggers
